"""
    A Report plugin to send a report to another host using SCP.
    Copyright (C) 2009 Red Hat, Inc

    Author: Gavin Romig-Koch <gavin@redhat.com>

    Much of the code in this module was derived from code written by
    Chris Lumens <clumens@redhat.com> and Will Woods <wwoods@redhat.com>.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import os
import pty

import gettext
_ = lambda x: gettext.ldgettext("report", x)

label="SCP"
description="scp problem to a given host"

def report(signature, io):
    attachedFileName = None

    if signature.has_key("pythonUnhandledException"):
        attachedFileName = signature["pythonUnhandledException"].asFileName()

    elif signature.has_key("description"):
        attachedFileName = signature["description"].asFileName()

    elif signature.has_key("signature"):
        attachedFileName = signature["signature"].asFileName()
    else:
        raise ValueError("signature contains neither a 'signature', 'description', nor a 'pythonUnhandledException'")

    return copyFileToRemote(attachedFileName, io)


def scpAuthenticate(master, childpid, password):
    while True:
        # Read up to password prompt.  Propagate OSError exceptions, which
        # can occur for anything that causes scp to immediately die (bad
        # hostname, host down, etc.)
        buf = os.read(master, 4096)
        if buf.lower().find("password: ") != -1:
            os.write(master, password+"\n")
            # read the space and newline that get echoed back
            os.read(master, 2)
            break

    while True:
        buf = ""
        try:
            buf = os.read(master, 4096)
        except (OSError, EOFError):
            break

    (pid, childstatus) = os.waitpid (childpid, 0)
    return childstatus

def copyFileToRemote(exnFileName, io):

    host = io.queryField("host")
    if not host or host.strip() == "":
        io.failMessage(_("No Host"),
                       _("Please provide a valid hostname"))
        return False
        
    if host.find(":") != -1:
        (host, port) = host.split(":")

        # Try to convert the port to an integer just as a check to see
        # if it's a valid port number.  If not, they'll get a chance to
        # correct the information when scp fails.
        try:
            int(port)
            portArgs = ["-P", port]
        except ValueError:
            portArgs = []
    else:
        portArgs = []

    loginResult = io.queryLogin(host)
    if loginResult:
        username = loginResult['username']
        password = loginResult['password']
    else:
        io.failMessage(_("Login Input Failed"),
                       _("Please provide a valid username and password"))
        return False

    path = io.queryField("path")
    if not path or path.strip() == "":
        io.failMessage(_("No Path"),
                       _("Please provide a path"))
        return False
        
    target = "%s@%s:%s" % (username, host, path)

    # Fork ssh into its own pty
    (childpid, master) = pty.fork()
    if childpid < 0:
        raise RuntimeError("Could not fork process to run scp")
    elif childpid == 0:
        # child process - run scp
        args = ["scp", "-oNumberOfPasswordPrompts=1",
                "-oStrictHostKeyChecking=no"] + portArgs + \
               [exnFileName, target]
        os.execvp("scp", args)

    # parent process
    try:
        childstatus = scpAuthenticate(master, childpid, password)
    except OSError:
        io.failMessage(_("scp failed"),
                       _("failed to scp file"))
        return False

    os.close(master)

    if os.WIFEXITED(childstatus) and os.WEXITSTATUS(childstatus) == 0:
        io.updateLogin(host,loginResult)
        io.successMessage(_("scp Successful"),
                          _("The signature was successfully copied to:"),
                          None, target)
        return True
    else:
        io.failMessage(_("scp failed"),
                       _("unexpected child status"))
        return False

